/**
 * Copyright (c) 2012 by sangame.com.
 * All right reserved.
 */
package com.lmsoft.cornu.webtrade.sdk;

import java.util.List;

import com.lmsoft.cornu.webtrade.sdk.message.all.MB10032;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10042;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13512;
import com.lmsoft.cornu.webtrade.sdk.message.bean.QueryType;

/**
 * 所有client都需要实现的接口 <BR>
 * @author Jerry Zhang <BR>
 * 2012-12-27 下午05:19:10
 */
public interface IBaseSkyFallClient {
	/**
	 * 发起连接请求
	 * @throws Exception
	 */
	abstract void connect() throws Exception ;
	/**
	 * 断开连接
	 * @throws Exception
	 */
	abstract void disconnect() throws Exception ;
	
	public boolean isConnected();
	
	void setCommHandler(CommHandler commHandler);
	/**
	 * 对时
	 * @return 返回服务器的时间（毫秒） 
	 */
	abstract public long timeSyn();
	/**
	 * 登陆挑战
	 * @param loginName 用户名/CustId
	 * @return MB10032
	 */
	abstract public MB10032 loginChallenge(String loginName);
	/**
	 * 登出
	 */
	abstract public void logOut();
	/**
	 * 心跳
	 */
	abstract public void heartBeat();
	/**
	 * 登陆
	 * @param loginName 用户名
	 * @param password 密码
	 * @param ip ip地址
	 * @return MB10042
	 */
	abstract public MB10042 login(String loginName, String password, String ip);
	/**
	 * 商品查询, 针对skyfall v2.0 添加
	 * @param types 
	 * @return MB13512
	 * @see com.lmsoft.cornu.webtrade.sdk.message.all.MB13471#queryTypes
	 * @since skyfall v2.0
	 */
	abstract public MB13512 getTradeSymbolsForSkyfall2(List<QueryType> types);
}
