/**
 * Copyright (c) 2012 by sangame.com.
 * All right reserved.
 */
package com.lmsoft.cornu.webtrade.sdk;

import com.lmsoft.cornu.webtrade.sdk.message.all.*;
import com.lmsoft.cornu.webtrade.sdk.message.bean.*;

import java.util.List;

/**
 * 所有client都需要实现的接口 <BR>
 * @author Jerry Zhang <BR>
 * 2012-12-27 下午05:19:10
 */
public interface SkyFallClient extends IBaseSkyFallClient{
	/**
	 * 市价买单请求
	 * @param askId  请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openBuyHolding(int askId, TradeOrderPost to);
	/**
	 * 市价卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openSellHolding(int askId, TradeOrderPost to);
	/**
	 * 限价挂买单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openBuyPending(int askId, TradeOrderPost to);
	/**
	 * 限挂卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openSellPending(int askId, TradeOrderPost to);
	/**
	 * 到价挂买单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openBuyStopPending(int askId, TradeOrderPost to);
	/**
	 * 到价挂卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 openSellStopPending(int askId, TradeOrderPost to);
	
	/**
	 * TD模式的挂买单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 * @since skyfall v1.1
	 */
	abstract public MB20012 openTdBuyPending(int askId, TradeOrderPost to);
	/**
	 * TD模式的挂卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 * @since skyfall v1.1
	 */
	abstract public MB20012 openTdSellPending(int askId, TradeOrderPost to);
	/**
	 * TD模式的挂买单平单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 * @since skyfall v1.1
	 */
	abstract public MB20012 openTdBuyClosePending(int askId, TradeOrderPost to);
	/**
	 * TD模式的限挂卖单平单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 * @since skyfall v1.1
	 */
	abstract public MB20012 openTdSellClosePending(int askId, TradeOrderPost to);
	/**
	 * 持仓汇总查询请求
	 * @param accntId 账户Id
	 * @return MB13502
	 * @since skyfall v1.1
	 */
	abstract public MB13502 getHoldingOrderStatistics(int accntId);
	/**
	 * 持仓汇总查询请求
	 * @param cngoldId cngoldId
	 * @return MB13502
	 * @since skyfall v1.1
	 */
	abstract public MB13502 getHoldingOrderStatisticsByCngoldId(int cngoldId);
	/**
	 * 取消请求
	 * @param askId 请求ID
	 * @param accntId 账户Id
	 * @return MB20012
	 */
	abstract public MB20012 cancelLastTrade(int askId, int accntId);
	/**
	 * 平买单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 closeBuyHolding(int askId, TradeOrderPost to);
	/**
	 * 平卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 closeSellHolding(int askId, TradeOrderPost to);
	/**
	 * 平卖单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20312 closeSellHoldingWithCngoldId(int askId, TradeOrderPost to);
	/**
	 * 修改持单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 modifyHolding(int askId, TradeOrderPost to);
	/**
	 * 修改挂单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 modifyPending(int askId, TradeOrderPost to);
	abstract public MB20312 modifyPendingWithCngoldId(int askId, TradeOrderPost to);
	/**
	 * 取消挂单请求
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 cancelPending(int askId, TradeOrderPost to);
	/**
	 * 与平
	 * @param askId 请求ID
	 * @param to 交易单
	 * @param to2  交易单
	 * @return MB20012
	 */
	abstract public MB20012 closeWith(int askId, TradeOrderPost to, TradeOrderPost to2);
	/**
	 * 对平
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 closeBy(int askId, TradeOrderPost to);
	/**
	 * 客户信息查询
	 * @param custId 客户ID
	 * @return MB13102
	 */
	abstract public MB13102 getCustInfo(int custId);
	/**
	 * 客户信息查询
	 * @param cngoldId cngoldId
	 * @return MB13102
	 */
	abstract public MB13102 getCustInfoByCnGoldId(int cngoldId);
	/**
	 * 客户组市场查询请求
	 * @param groupId 客户组ID
	 * @return MB13082
	 */
	abstract public MB13082 getGroupMarkets(int groupId);
	/**
	 * 基本信息查询请求
	 * @param key
	 * @return MB13012
	 */
	abstract public MB13012 getSysConfig(String key);
	/**
	 * 公告查询请求
	 * @return MB10012
	 */
	abstract public MB10012 getNotice();
	/**
	 * 续约
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 extensionHolding(int askId, TradeOrderPost to);
	/**
	 * 买单交割
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 deliveryBuyHolding(int askId, TradeOrderPost to);
	/**
	 * 卖单交割
	 * @param askId 请求ID
	 * @param to 交易单
	 * @return MB20012
	 */
	abstract public MB20012 deliverySellHolding(int askId, TradeOrderPost to);
	/**
	 * 系统银行账户查询请求, 用于出入金
	 * @return MB13352
	 */
	abstract public MB13352 getSysAccnts();
	/**
	 * 交易账户查询请求
	 * @param custId 客户ID
	 * @return MB13312
	 */
	abstract public MB13312 getTradeAccnts(int custId);
	
	/**
     * 根据cngoldId查询账户信息
     * */
	abstract MB13622 getTradeAccntsByCnGoldId(int cngoldId);
	
	/**
	 * 查所有行情
	 * @return MB12062
	 */
	abstract public MB12062 queryAllQuoto();
	/**
	 * 交易单查询请求
	 * @param custId 客户ID
	 * @param queryTypes 查询条件列表
	 * @param queryTimeType 时间起止
	 * @return MB13332
	 */
	abstract public MB13332 getTradeOrders(int custId, List<QueryType> queryTypes, QueryTimeType queryTimeType);

	/**
	 * 出入金请求
	 * @param custId
	 * @param accntId
	 * @param bankId
	 * @param reqType
	 * @param balance
	 * @param reqReason
	 * @return MB10992
	 */
	abstract public MB10992 balanceReq(int custId, int accntId, int bankId, int reqType, double balance, String reqReason);
	/**
	 * 商品查询, 因为3.2的不支持指定商品的查询, 所以3.3新增一协议用以实现指定商品查询
	 * @param types 
	 * @return MB13032
	 * @see com.lmsoft.cornu.webtrade.sdk.message.all.MB13471#queryTypes
	 */
	abstract public MB13032 getTradeSymbolsFor3_3(List<QueryType> types);
	/**
	 * 股票行情查询
	 * @param stockIdList 股票Id列表
	 * @return MB12153
	 */
	abstract public MB12153 getStockList(List<Integer> stockIdList);
	/**
	 * 股票商品标签查询请求
	 * @return MB13462
	 */
	abstract public MB13462 getStockTag();
	/**
	 * 行情开关
	 * @param switches 开关列表
	 * @return MB12052
	 */
	abstract public MB12052 symbolSwitch(List<SymbolSwitch> switches);
	/**
	 * 修改密码
	 * @param custId 客户ID
	 * @param oldPwd 原密码
	 * @param newPwd 新密码
	 * @return MB10992
	 */
	abstract public MB10992 modifyPassword(int custId, String oldPwd, String newPwd);
	/**
	 * 登陆确认
	 * @param quotoPost 商品行情请求
	 * @return MB12012
	 */
	abstract public MB12012 queryQuotoPost(QuotoPost quotoPost);
	/**
	 * 商品交易控制查询
	 * @param idList 商品ID列表
	 * @return MB13452
	 */
	abstract public MB13452 getSymbolOpts(List<Integer> idList);
	
	/**
	 * 客户组信息查询
	 * @param groupId 客户组ID
	 * @return 
	 * @since skyfall v2.0
	 */
	abstract public MB13522 getCustGroup(int groupId);
	
	abstract public MB15992 custReset(int custId);

	MB10042 getLoginTokenByCngoldId(int i);
}
