/**
 * Copyright (c) 2012 by sangame.com.
 * All right reserved.
 */
package com.lmsoft.cornu.webtrade.sdk.client2;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.lmsoft.cornu.webtrade.sdk.IBaseSkyFallClient;
import com.lmsoft.cornu.webtrade.sdk.TidConst;
import com.lmsoft.cornu.webtrade.sdk.client2.phoenix.ComminicationClient;
import com.lmsoft.cornu.webtrade.sdk.config.WebTraderConfig;
import com.lmsoft.cornu.webtrade.sdk.message.BaseWebTraderField;
import com.lmsoft.cornu.webtrade.sdk.message.MutiPackage;
import com.sangame.phoenix.cornu.CornuMessage;

/**
 * 所有客户端都通用锁的写法 <BR>
 * @author Jerry Zhang <BR>
 * 2012-12-28 上午09:23:32
 */
public abstract class BaseSkyfallCommClient extends BaseCommClient implements IBaseSkyFallClient{
	private static final Log log = LogFactory.getLog(BaseSkyfallCommClient.class);
	
	
	//////////////////////////////////////////////////////////////////////
	protected static int TIME_OUT_TIME = 10;
	/**
	 * 对话模式的锁，用于防止同时发送过多的消息
	 * 除心跳不锁外，其它全锁
	 */
	protected ReentrantLock dialogLock = new ReentrantLock();
	protected final Condition sending  = dialogLock.newCondition();
	
	
	protected BaseWebTraderField nowCornuMessage = null;
	protected List<BaseWebTraderField> mutiPackageCache = new ArrayList<BaseWebTraderField>();
	
	protected long returnTime = 0;
	
	protected int nowDialogSeq;

	
	public BaseSkyfallCommClient(WebTraderConfig config) {
		PrivateMsgTheadPool privateMsgTheadPool = PrivateMsgTheadPool.getInstance();
		comminicationClient = new ComminicationClient(config, this, this, this, privateMsgTheadPool);
	}
	
	///////////////////////////////////CommClient////////////////////////////////
	public void connect() throws Exception {
		comminicationClient.init();
		comminicationClient.connect();
	}
	
	public void disconnect() throws Exception {
		comminicationClient.disconnect();
	}

	@Override
	public boolean isConnected() {
		return comminicationClient.haveConnected();
	}
	
	protected void sendMsgAndSetNoSeq(int tid, BaseWebTraderField bwtf){
		nowDialogSeq = sendMsgAndSetAliveMills(tid, bwtf);
	}
	
	private int sendMsgAndSetAliveMills(int tid, BaseWebTraderField bwtf){
		setLastAliveMills(System.currentTimeMillis());
		return sendMsg(tid, bwtf);
	}
	
	/////////////////////////////dialog////////////////////////////
	public void receiveMsg(CornuMessage msg) {
		dialogLock.lock();
		try {
			//心跳不做任何处理
			if(msg.getTid() == TidConst.T10141)
				return;
			// 如果seq号对应的不对, 说明不是当前想要的回包, 做丢弃处理
			if(msg.getSeqNo()!=nowDialogSeq)
				return;
			if(msg.getChain()==CornuMessage.CHAIN_SINGLE){	// single的直接发送信号量并返回
				nowCornuMessage = (BaseWebTraderField)msg.getFields()[0];
				sending.signal();
			}else{											// 不是single的要进行多包的暂存
				switch (msg.getChain()) {
				case CornuMessage.CHAIN_FIRST:				// 第一个包暂存
					mutiPackageCache.add((BaseWebTraderField)msg.getFields()[0]);
					break;
				case CornuMessage.CHAIN_CONTINUE:			// 中间的包暂存
					mutiPackageCache.add((BaseWebTraderField)msg.getFields()[0]);
					break;
				case CornuMessage.CHAIN_LAST:				// 最后一个包进行组包, 清cache, 解锁
					mutiPackageCache.add((BaseWebTraderField)msg.getFields()[0]);		// 暂存
					if(msg.getFields()[0] instanceof MutiPackage){
						nowCornuMessage = ((MutiPackage)msg.getFields()[0]).packageing(mutiPackageCache);// 组包
					}else{
						// 如果服务器已经分包，但客户端没有分包，则只返回第一个包
						nowCornuMessage = mutiPackageCache.get(0);
					}
					mutiPackageCache.clear();				// 清缓存
					sending.signal();						// 发送信号量
					break;
				default:
					throw new IllegalStateException("怎么可能还有第五种包结构????" + msg.getChain());
				}
			}
		}catch (Exception e) {
			log.error("error", e);
		} finally {
			dialogLock.unlock();
		}
	}

	
}
