/**
 * Copyright (c) 2012 by sangame.com.
 * All right reserved.
 */
package com.lmsoft.cornu.webtrade.sdk.client2;

import java.util.List;
import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.lmsoft.cornu.webtrade.sdk.IBaseSkyFallClient;
import com.lmsoft.cornu.webtrade.sdk.TidConst;
import com.lmsoft.cornu.webtrade.sdk.config.WebTraderConfig;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10031;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10032;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10061;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10141;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB12061;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB12062;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13032;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13332;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13361;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13471;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB13512;
import com.lmsoft.cornu.webtrade.sdk.message.bean.QueryTimeType;
import com.lmsoft.cornu.webtrade.sdk.message.bean.QueryType;

/**
 * Skyfall通用的业务协议实现, serverclient , manager, cust client <BR>
 * @author Jerry Zhang <BR>
 * 2012-12-28 上午11:26:44
 */
public abstract class BaseSkyfallCommonBizClient extends BaseSkyfallCommClient implements IBaseSkyFallClient{
	private static final Log log = LogFactory.getLog(BaseSkyfallCommonBizClient.class);

	public BaseSkyfallCommonBizClient(WebTraderConfig config) {
		super(config);
	}

	public void heartBeat() {
		MB10141 mb10141 = new MB10141();
		sendMsg(TidConst.T10141, mb10141);
	}
	
	public void logOut() {
		MB10061 mb10061 = new MB10061();
		sendMsg(TidConst.T10061, mb10061);
	}
	
	public long timeSyn() {
		return System.currentTimeMillis();
	}
	

	public synchronized MB10032 loginChallenge(String loginName) {
		dialogLock.lock();
		MB10032 mb10032 = null;
		try {
			MB10031 mb10031 = new MB10031(loginName);
			nowDialogSeq = 0;
			sendUnloginMsg(TidConst.T10031, mb10031, nowDialogSeq);

			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			
			if(nowCornuMessage ==null){
				
			}else
				mb10032 = (MB10032)nowCornuMessage;
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}
		return mb10032;
	}

	public synchronized MB12062 queryAllQuoto() {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB12062 mb12062 = null;
		try {
			MB12061 mb12061 = new MB12061();
			sendMsgAndSetNoSeq(TidConst.T12061, mb12061);

			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else
				mb12062 = (MB12062)nowCornuMessage;

		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}
		return mb12062;
	}

	public synchronized MB13332 getTradeOrders(int custId, List<QueryType> queryTypes, QueryTimeType queryTimeType) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB13332 mb13332 = null;
		try {
			MB13361 mb13361 = new MB13361(custId, queryTypes, queryTimeType);
			sendMsgAndSetNoSeq(TidConst.T13361, mb13361);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);

			if(nowCornuMessage ==null){
				
			}else
				mb13332 = (MB13332)nowCornuMessage;
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb13332;
	}
	
	public synchronized MB13332 getTradeOrdersWithCngoldId(int custId, List<QueryType> queryTypes, QueryTimeType queryTimeType) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB13332 mb13332 = null;
		try {
			MB13361 mb13361 = new MB13361(custId, queryTypes, queryTimeType);
			sendMsgAndSetNoSeq(TidConst.T13661, mb13361);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);

			if(nowCornuMessage ==null){
				
			}else
				mb13332 = (MB13332)nowCornuMessage;
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb13332;
	}
	
	public synchronized MB13032 getTradeSymbolsFor3_3(List<QueryType> types) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB13032 mb13032 = null;
		try {
			MB13471 mb13471 = new MB13471(types);
			sendMsgAndSetNoSeq(TidConst.T13471, mb13471);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);

			if(nowCornuMessage ==null){
				
			}else
				mb13032 = (MB13032)nowCornuMessage;

		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb13032;
	}

	@Override
	public MB13512 getTradeSymbolsForSkyfall2(List<QueryType> types) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB13512 mb13032 = null;
		try {
			MB13471 mb13471 = new MB13471(types);
			sendMsgAndSetNoSeq(TidConst.T13611, mb13471);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);

			if(nowCornuMessage ==null){
				
			}else
				mb13032 = (MB13512)nowCornuMessage;

		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}
		return mb13032;
	}
}
