package com.lmsoft.cornu.webtrade.sdk.client2;

import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.lmsoft.cornu.webtrade.sdk.TidConst;
import com.lmsoft.cornu.webtrade.sdk.VoucherTradeClient;
import com.lmsoft.cornu.webtrade.sdk.config.WebTraderConfig;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB10992;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB20012;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB20311;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB20312;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB20331;
import com.lmsoft.cornu.webtrade.sdk.message.all.MB20341;
import com.lmsoft.cornu.webtrade.sdk.message.bean.TradeOrderPost;

/**
 * 金元券业务
* @author liuming
* @date 2016年11月17日
 */
public abstract class BaseSkyfallVoucherBizClient extends BaseSkyfallBizClient implements VoucherTradeClient {
	private static final Log log = LogFactory.getLog(BaseSkyfallVoucherBizClient.class);

	public BaseSkyfallVoucherBizClient(WebTraderConfig config) {
		super(config);
	}

	@Override
	public MB20312 openBuyHoldingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20311 mb20311 = new MB20311(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getMaxWarp(), to.getOpen_prc(), to.getMemo(), to.getVouchers(),(short)1);
			sendMsgAndSetNoSeq(TidConst.T20311, mb20311);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
					mb20312.setOpen_prc(mb20012.getPrice());
					mb20312.setSmbl_id(mb20012.getSymbolId());
					mb20312.setLots(mb20012.getLots());
					mb20312.setOrder_type(0);
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}
	
	@Override
	public MB20312 openSellHoldingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20311 mb20311 = new MB20311(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getMaxWarp(), to.getOpen_prc(), to.getMemo(), to.getVouchers(),(short)2);
			sendMsgAndSetNoSeq(TidConst.T20321, mb20311);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
					mb20312.setOpen_prc(mb20012.getPrice());
					mb20312.setSmbl_id(mb20012.getSymbolId());
					mb20312.setLots(mb20012.getLots());
					mb20312.setOrder_type(1);
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}
	
	@Override
	public synchronized MB20312 openTdBuyPendingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20331 mb20331 = new MB20331(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getOpen_prc(), to.getMemo(), 
					to.getExpire_day()==null ? 0 : to.getExpire_day().getTime(), to.getVouchers());
			sendMsgAndSetNoSeq(TidConst.T20351, mb20331);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}
	
	@Override
	public synchronized MB20312 openTdSellPendingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20331 mb20331 = new MB20331(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getOpen_prc(), to.getMemo(), 
					to.getExpire_day()==null ? 0 : to.getExpire_day().getTime(), to.getVouchers());
			sendMsgAndSetNoSeq(TidConst.T20361, mb20331);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}
	
	@Override
	public synchronized MB20312 openTdBuyClosePendingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20331 mb20331 = new MB20331(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getOpen_prc(), to.getMemo(), 
					to.getExpire_day()==null ? 0 : to.getExpire_day().getTime(), "");
			sendMsgAndSetNoSeq(TidConst.T20371, mb20331);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}

	@Override
	public synchronized MB20312 openTdSellClosePendingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20331 mb20331 = new MB20331(askId, to.getCngold_id(), (int)to.getSmbl_id(), to.getLots(), 
					to.getProfit_prc(), to.getLoss_prc(), to.getDraft_point(), to.getOpen_prc(), to.getMemo(), 
					to.getExpire_day()==null ? 0 : to.getExpire_day().getTime(), "");
			sendMsgAndSetNoSeq(TidConst.T20381, mb20331);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
				}else if(nowCornuMessage instanceof MB10992){
					MB10992 mb10992 = (MB10992) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb10992.getRtnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}
	
	@Override
	public synchronized MB20312 cancelPendingByVouchers(int askId, TradeOrderPost to) {
		//以下一段话必带， 用于锁对话模式
		dialogLock.lock();
		MB20312 mb20312 = null;
		try {
			MB20341 mb20341 = new MB20341(askId, to.getCngold_id(), to.getSmbl_id(), to.getId());
			sendMsgAndSetNoSeq(TidConst.T20341, mb20341);
			
			//以下一段话必带，用于等对话模式回包
			sending.await(TIME_OUT_TIME, TimeUnit.SECONDS);
			if(nowCornuMessage ==null){
				
			}else{
				if(nowCornuMessage instanceof MB20012){
					MB20012 mb20012 = (MB20012) nowCornuMessage;
					mb20312 = new MB20312();
					mb20312.setRtnCode(mb20012.getReturnCode());
				}else{
					mb20312 = (MB20312)nowCornuMessage;
				}
			}
		} catch (Exception e) {
			log.error("error", e);
		} finally{
			nowCornuMessage = null;
			dialogLock.unlock();
		}

		return mb20312;
	}

}
