/**
 * 
 */
package com.lmsoft.cornu.webtrade.sdk.client2;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ThreadPoolExecutor.AbortPolicy;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * @author JerryZ
 *
 */
public class PrivateMsgTheadPool {

	private static final Log log = LogFactory.getLog(PrivateMsgTheadPool.class);

	// 池中所保存的线程数，包括空闲线程
	private final static int CORE_POOL_SIZE = Runtime.getRuntime().availableProcessors();

	// 池中允许的最大线程数
	private static int MAX_POOL_SIZE = Runtime.getRuntime().availableProcessors() * 8;

	// 当线程数大于核心时，此为终止前多余的空闲线程等待新任务的最长时间
	private final static int KEEP_ALIVE_TIME = 3;

	// 执行前用于保持任务的队列。此队列仅保持由 execute 方法提交的 Runnable 任务
	private final static int WORK_QUEUE_SIZE = 3000;

	// 线程池
	private  ThreadPoolExecutor threadPool = null;

	// 
	private static long allExecuteTime = 0;
	
	private static long allExecuteTimes = 0;
	
	private static long beginTime = 0;
	
	private static long allWaitTime = 0;
	
	private static long maxExecuteTime = 0;
	
	public static int maxExecuteTid = 0;
	
	private  static PrivateMsgTheadPool privateMsgTheadPool = null;

	public static PrivateMsgTheadPool getInstance() {
		if(privateMsgTheadPool==null){
			privateMsgTheadPool=new PrivateMsgTheadPool();
		}
		return privateMsgTheadPool;
	}

	private PrivateMsgTheadPool() {
		threadPool = new ThreadPoolExecutor(CORE_POOL_SIZE, MAX_POOL_SIZE,
				KEEP_ALIVE_TIME, TimeUnit.SECONDS,
				new ArrayBlockingQueue<Runnable>(WORK_QUEUE_SIZE),
				new DefaultPrivateMsgThreadFactory(),
				// 由于超出线程范围和队列容量而使执行被阻塞时所使用的处理程序
				new AbortPolicy()) {

			@Override
			protected void afterExecute(Runnable r, Throwable t) {
				long exetime = System.currentTimeMillis() - beginTime;
				if(exetime > maxExecuteTime){
					maxExecuteTime = exetime;
					maxExecuteTid = ((PrivateMsgTask)r).msg.getTid();
				}
				allExecuteTime += exetime;
				allExecuteTimes ++ ;
				if(log.isDebugEnabled()){
					if(allExecuteTimes % 1000 == 1){
						/*log.debug("PrivateMsgTheadPool : 执行次数:" +
								allExecuteTimes + "总执行时间:" + allExecuteTime + "总等待时间为" + allWaitTime
								+ "单个任务执行的峰值为" + maxExecuteTime + "峰值Tid:" + maxExecuteTid
								);*/
					}
				}
					
				super.afterExecute(r, t);
			}

			@Override
			protected void beforeExecute(Thread t, Runnable r) {
				beginTime = System.currentTimeMillis();
				allWaitTime += beginTime - ((PrivateMsgTask)r).getEnterTime();
				super.beforeExecute(t, r);
			}
		};
	}

	/**
	 * The default EPC thread factory
	 */
	static class DefaultPrivateMsgThreadFactory implements ThreadFactory {
		static final AtomicInteger poolNumber = new AtomicInteger(1);
		final ThreadGroup group;
		final AtomicInteger threadNumber = new AtomicInteger(1);
		final String namePrefix;

		DefaultPrivateMsgThreadFactory() {
			SecurityManager s = System.getSecurityManager();
			group = (s != null) ? s.getThreadGroup() : Thread.currentThread()
					.getThreadGroup();
			namePrefix = "PrivateMsg-" + poolNumber.getAndIncrement() + "-worker-";
		}

		public Thread newThread(Runnable r) {
			Thread t = new Thread(group, r, namePrefix
					+ threadNumber.getAndIncrement(), 0);
			if (t.isDaemon())
				t.setDaemon(false);
			if (t.getPriority() != Thread.NORM_PRIORITY)
				t.setPriority(Thread.NORM_PRIORITY);
			return t;
		}
	}

	private static AtomicLong _discard = new AtomicLong(0);

	public void addPrivateMsgTask(PrivateMsgTask privateMsgTask) {
		try {
			privateMsgTask.setEnterTime(System.currentTimeMillis());
			threadPool.execute(privateMsgTask);
		} catch (RejectedExecutionException ree) {
			log.error("ThreadPool is full, discard the[ "
					+ _discard.incrementAndGet() + " ] trade:" + privateMsgTask);
		}
	}
}
