/**
 * 
 */
package com.lmsoft.cornu.webtrade.sdk.client2.phoenix;

import javax.net.ssl.SSLContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.mina.core.filterchain.DefaultIoFilterChainBuilder;
import org.apache.mina.core.service.IoConnector;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.ssl.SslFilter;
import org.apache.mina.transport.socket.nio.NioSocketConnector;

import com.lmsoft.cornu.webtrade.sdk.message.WebTraderFieldFactory;
import com.sangame.phoenix.client.ClientConfig;
import com.sangame.phoenix.client.PhoenixClient2;
import com.sangame.phoenix.client.PhoenixClientHandler;
import com.sangame.phoenix.codec.PhoenixProtocolCodecFactory;
import com.sangame.phoenix.filter.DealExceptionFilter;
import com.sangame.phoenix.filter.LogAfterCodec;
import com.sangame.phoenix.filter.LogBeforeCodec;
import com.sangame.phoenix.filter.StatisticFilter;
import com.sangame.phoenix.filter.ssl.BogusSslContextFactory;

/**
 * @author JerryZ
 * 因为一个SDK只需要一个SC, 所以SC做为单例的统一进行初始化
 */
public class SingleSc {
	private static final Log log = LogFactory.getLog(PhoenixClient2.class);

	private static SingleSc self;
	IoConnector sc;
	
	private SingleSc() {
		ClientConfig config = new ClientConfig();
		
		sc = new NioSocketConnector();
		
        sc.getFilterChain().addLast("Stat.", new StatisticFilter());
        
        /*if (config.getMaxReadThroughputPerSession() > 0 || config.getMaxWriteThroughputPerSession() > 0)
        	sc.getFilterChain().addFirst("traffic", 
        		new TrafficShapingFilter(Executors.newScheduledThreadPool(Runtime.getRuntime().availableProcessors()),
        				config.getMaxReadThroughputPerSession(), config.getMaxWriteThroughputPerSession()));
        */
        if (config.enableLogBeforeDecodec()) {
        	sc.getFilterChain().addLast( "DEBUG", new LogBeforeCodec() );
        	log.info("LogBeforeDecodec is enabled ......");
        }
        
        if (config.enableSsl()) {
        	addSSLSupport(sc.getFilterChain());
        }
        
        sc.getFilterChain().addLast("codec",
                new ProtocolCodecFilter( new PhoenixProtocolCodecFactory(new WebTraderMessageFactory(), new WebTraderFieldFactory()) ) );  
        sc.getFilterChain().addLast("dealException", new DealExceptionFilter());
        
        if (config.enableLogAfterDecodec()) {
        	sc.getFilterChain().addLast( "logger", new LogAfterCodec() );
        	log.info("LogAfterDecodec is enabled ......");
        }
        
        sc.setHandler(new PhoenixClientHandler(new WebTraderBizMessageHandler(), new WebTraderSessionEventHandler(), new WebTraderMessageHandler()));

	} 
	
	private void addSSLSupport(DefaultIoFilterChainBuilder filterChain) {
		try {
	        SSLContext sslContext = BogusSslContextFactory.getInstance(false);
			SslFilter sslFilter = new SslFilter(sslContext);
			sslFilter.setUseClientMode(true);
			filterChain.addLast("sslFilter", sslFilter);
			
			log.info("SSL is enabled.....");
		} catch (Exception ex) {
			log.warn("SSL enable failed, cause:", ex);
		}
	}
	
	public synchronized static SingleSc getInstance(){
		if(self == null){
			self = new SingleSc();
		}
		return self;
	}

	public IoConnector getSc() {
		return sc;
	}
}
