/**
 * Copyright (c) 2008 by sangame.com.
 * All right reserved.
 */
package com.lmsoft.cornu.webtrade.sdk.config;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sangame.phoenix.client.ClientConfig;

/**
 * web trade系统配置
 * 
 * @author Robin Zhou $Id$
 */
public class WebTraderConfig extends ClientConfig {
    private static final Log log = LogFactory.getLog(WebTraderConfig.class);

    private String serverVersion;
    
    private int serverType;
    
    private String serverIp;
    
	private int serverPort;

	private boolean enablessl;
	
	private boolean enableReg;
	
	private String systemType;
	
	private int languageType;

	private Set<String> blacklist;
	
	private String companyName;
	
	private String telphone;
	
	private String systemName;
	
	private String siteUtl;
	
	private int chartWidth;
	
	private int chartHeight;
	
	private boolean needMoveSl;
	
	private static WebTraderConfig webTraderConfig;
	
	private WebTraderConfig(){
		loadConfigXml();
	}
	
	public WebTraderConfig(String ip, int port, String version){
		serverIp = ip;
		serverPort = port;
		serverVersion = version;
	}
	
	
	public static WebTraderConfig getInstance(){
		if(webTraderConfig==null){
			webTraderConfig=new WebTraderConfig();
		}
		return webTraderConfig;
	}
	
	protected void loadConfigXml() {
		try {
			XMLConfiguration config = new XMLConfiguration("web-trader.xml");

			System.out.println("Loaded config from file '"
					+ config.getFile().getAbsolutePath() + "'.");

			serverVersion = config.getString("client.server-version");

			serverType = config.getInt("client.server-type");
			
			serverPort = config.getInt("client.server-port");

			serverIp =  config.getString("client.server-ip");
			
			enablessl = config.getBoolean("client.enable-ssl");
			
			enableReg = config.getBoolean("client.enable-reg");
			
			systemType = config.getString("client.system-type");
			
			languageType = config.getInt("client.language-type");
			
			needMoveSl = config.getBoolean("client.needMoveSl");

			String[] bl = config
					.getStringArray("server.access-address.blacklist.ip");
			
			companyName = config.getString("company.company-name");
			
			telphone = config.getString("company.telphone");
			
			systemName = config.getString("company.system-name");
			
			siteUtl = config.getString("company.site-url");
			
			chartWidth=config.getInt("chart.chart-width");
			
			chartHeight=config.getInt("chart.chart-height");
			
			blacklist = new HashSet<String>();
			if (bl != null) {
				for (int i = 0; i < bl.length; i++) {
					blacklist.add(bl[i]);
				}
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			log.error("Load config error.", e);
			System.exit(0);
		}
	}
	
	public String getServerVersion() {
		return serverVersion;
	}

	public int getServerType() {
		return serverType;
	}

	public int getChartWidth() {
		return chartWidth;
	}
	
	public void setChartWidth(int chartWidth) {
		this.chartWidth = chartWidth;
	}

	public int getChartHeight() {
		return chartHeight;
	}

	public String getSiteUtl() {
		return siteUtl;
	}

	public int getLanguageType() {
		return languageType;
	}

	public String getSystemType() {
		return systemType;
	}

	public String getServerIp() {
		return serverIp;
	}

	public int getServerPort() {
		return serverPort;
	}

	public boolean enableLogBeforeDecodec() {
		return true;
	}

	public boolean enableLogAfterDecodec() {
		return false;
	}
	
	public boolean enableSsl() {
		return enablessl;
	}

	public Set<String> getBlacklist() {
		return blacklist;
	}

	public String getCompanyName() {
		return companyName;
	}

	public String getTelphone() {
		return telphone;
	}

	public String getSystemName() {
		return systemName;
	}

	@Override
	public boolean isAutoReconnect() {
		return false;
	}

	public boolean isEnableReg() {
		return enableReg;
	}

	public boolean isNeedMoveSl() {
		return needMoveSl;
	}
}
