/**
 * 
 */
package com.lmsoft.cornu.webtrade.sdk.message.all;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.lmsoft.cornu.webtrade.sdk.PidConst;
import com.lmsoft.cornu.webtrade.sdk.message.BaseWebTraderField;
import com.lmsoft.cornu.webtrade.sdk.message.MutiPackage;

/**
 * 定单查询应答
 * @author JerryZ 
 * 2009-6-17 下午04:00:18
 *
 */
public class MB13332 extends BaseWebTraderField implements MutiPackage{
	
	List<MB13332> tradeOrders = new ArrayList<MB13332>();
	
	@Override
	protected void readCommandBody() {
		
		int size = readShort();
		for(int i=0; i<size; i++) {
			MB13332 traderOrder = new MB13332();
			traderOrder.setId(readInt());					// OrderId
			traderOrder.setAccnt_id(readInt());				// AccntId
			readString();									// AccountCrr
			readString();									// CustName
			traderOrder.setOrder_type(readByte());			// OrderType
			traderOrder.setOrder_status(readByte());		// OrderStatus
			traderOrder.setLots(readFloatD());				// Lots
			traderOrder.setSmbl_id(readInt());				// SmbolId
			traderOrder.setSymbol_name(readString());		// SmbolName
			long openTime = readLong();
			traderOrder.setOpen_time(openTime==0 ? null : new Date(openTime));
			long closeTime = readLong();
			traderOrder.setClose_time(closeTime==0 ? null : new Date(closeTime));
			traderOrder.setOpen_prc(readFloatD());			// OpenPrc
			traderOrder.setClose_prc(readFloatD());			// ClosePrc
			traderOrder.setLoss_prc(readFloatD());			// LossPrc
			traderOrder.setProfit_prc(readFloatD());		// ProfitPrc
			traderOrder.setTick_level((int)readFloatD());	// TickLevel
			traderOrder.setCommission(readFloatD());		// Commission
			traderOrder.setAgent_cms(readFloatD());			// AgentCms
			traderOrder.setInterest(readFloatD());			// Interest
			traderOrder.setProfit_loss(readFloatD());		// ProfitLoss
			traderOrder.setMemo(readString());				// Memo
			long expirday = readLong();						// ExpireDay
			traderOrder.setExpire_day(expirday==0 ? null : new Date(expirday));
			traderOrder.setDraft_prc(readFloatD());			// DraftPrc
			traderOrder.setDraft_point(readShort());		// DraftPoint
			traderOrder.setClose_type(readByte());			// CloseType
			long timeEffective = readLong();
			traderOrder.setTime_effective(timeEffective==0 ? null : new Date(timeEffective));
			
			tradeOrders.add(traderOrder);
		}
	}

	@Override
	protected void writeCommandBody() {}

	@Override
	public int getCommandCode() {
		return PidConst.P13332;
	}


	int id;
	
	/**
	 * 账号ID
	 */
	private int accnt_id;

	/**
	 * 商品ID
	 */
	private Integer smbl_id;
	/**
	 * 交易类型（9种，包括balance, credit）
	 * <ul>
	 * <li>0:买单包括持单和平单
	 * <li>1:卖单包括持单和平单
	 * <li>2:限价买单挂单,挂价比高价低
	 * <li>3:限价卖单挂单,挂价比低价高
	 * <li>4:止损买挂单,挂价比高价高
	 * <li>5:止损卖挂单,挂价比低价低
	 * <li>6:出入金
	 * <li>7:信用额度
	 * <li>8:调整金
	 * </ul>
	 */
	private int order_type;
	/**
	 * 单子所处状态(open,close)
	 */
	private int order_status;

	/**
	 * 交易数量
	 */
	private double lots;

	/**
	 * 开仓时间（建单时间，挂单时间）
	 */
	private Date open_time;

	/**
	 * 闭仓时间（平仓时间，挂单取消时间）
	 */
	private Date close_time;

	/**
	 * 开仓建
	 */
	private double open_prc;

	/**
	 * 平仓建
	 */
	private double close_prc;

	/**
	 * 平仓类型
	 */
	private int close_type;

	/**
	 * 止赢价
	 */
	private double profit_prc;

	/**
	 * 止损位
	 */
	private double loss_prc;
	/**
	 * 移动止损的现价
	 */
	private double draft_prc;
	/**
	 * 移动止损的点数
	 */
	private int draft_point;

	/**
	 * 盈亏
	 */
	private double profit_loss;

	/**
	 * 保证金水平
	 */
	private int tick_level;

	/**
	 * 手续费
	 */
	private double commission;

	/**
	 * 代理手续费(累加)
	 */
	private double agent_cms;

	/**
	 * 隔夜(累加)
	 */
	private double interest;

	/**
	 * 到期日
	 */
	private Date expire_day;

	/**
	 * 注释
	 */
	private String memo;

	/**
	 * 商品对保证金汇率
	 */
	private double first_con_rate;

	/**
	 * 保证金对帐户汇率
	 */
	private double second_con_rate;

	/**
	 * 持单时效(T+D专用)
	 */
	private Date time_effective;

	private String symbol_name;
	/**
	 * 最大偏差，用于建单与平单操作
	 */
	private int maxWarp;
	
	/**
	 * 剩余移动止损的点数
	 */
	private int left_draft_point;

	public List<MB13332> getTradeOrders() {
		return tradeOrders;
	}

	public void setTradeOrders(List<MB13332> tradeOrders) {
		this.tradeOrders = tradeOrders;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public int getAccnt_id() {
		return accnt_id;
	}

	public void setAccnt_id(int accntId) {
		accnt_id = accntId;
	}

	public Integer getSmbl_id() {
		return smbl_id;
	}

	public void setSmbl_id(Integer smblId) {
		smbl_id = smblId;
	}

	public int getOrder_type() {
		return order_type;
	}

	public void setOrder_type(int orderType) {
		order_type = orderType;
	}

	public int getOrder_status() {
		return order_status;
	}

	public void setOrder_status(int orderStatus) {
		order_status = orderStatus;
	}

	public double getLots() {
		return lots;
	}

	public void setLots(double lots) {
		this.lots = lots;
	}

	public Date getOpen_time() {
		return open_time;
	}

	public void setOpen_time(Date openTime) {
		open_time = openTime;
	}

	public Date getClose_time() {
		return close_time;
	}

	public void setClose_time(Date closeTime) {
		close_time = closeTime;
	}

	public double getOpen_prc() {
		return open_prc;
	}

	public void setOpen_prc(double openPrc) {
		open_prc = openPrc;
	}

	public double getClose_prc() {
		return close_prc;
	}

	public void setClose_prc(double closePrc) {
		close_prc = closePrc;
	}

	public int getClose_type() {
		return close_type;
	}

	public void setClose_type(int closeType) {
		close_type = closeType;
	}

	public double getProfit_prc() {
		return profit_prc;
	}

	public void setProfit_prc(double profitPrc) {
		profit_prc = profitPrc;
	}

	public double getLoss_prc() {
		return loss_prc;
	}

	public void setLoss_prc(double lossPrc) {
		loss_prc = lossPrc;
	}

	public double getDraft_prc() {
		return draft_prc;
	}

	public void setDraft_prc(double draftPrc) {
		draft_prc = draftPrc;
	}

	public int getDraft_point() {
		return draft_point;
	}

	public void setDraft_point(int draftPoint) {
		draft_point = draftPoint;
	}

	public double getProfit_loss() {
		return profit_loss;
	}

	public void setProfit_loss(double profitLoss) {
		profit_loss = profitLoss;
	}

	public int getTick_level() {
		return tick_level;
	}

	public void setTick_level(int tickLevel) {
		tick_level = tickLevel;
	}

	public double getCommission() {
		return commission;
	}

	public void setCommission(double commission) {
		this.commission = commission;
	}

	public double getAgent_cms() {
		return agent_cms;
	}

	public void setAgent_cms(double agentCms) {
		agent_cms = agentCms;
	}

	public double getInterest() {
		return interest;
	}

	public void setInterest(double interest) {
		this.interest = interest;
	}

	public Date getExpire_day() {
		return expire_day;
	}

	public void setExpire_day(Date expireDay) {
		expire_day = expireDay;
	}

	public String getMemo() {
		return memo;
	}

	public void setMemo(String memo) {
		this.memo = memo;
	}

	public double getFirst_con_rate() {
		return first_con_rate;
	}

	public void setFirst_con_rate(double firstConRate) {
		first_con_rate = firstConRate;
	}

	public double getSecond_con_rate() {
		return second_con_rate;
	}

	public void setSecond_con_rate(double secondConRate) {
		second_con_rate = secondConRate;
	}

	public Date getTime_effective() {
		return time_effective;
	}

	public void setTime_effective(Date timeEffective) {
		time_effective = timeEffective;
	}

	public String getSymbol_name() {
		return symbol_name;
	}

	public void setSymbol_name(String symbolName) {
		symbol_name = symbolName;
	}

	public int getMaxWarp() {
		return maxWarp;
	}

	public void setMaxWarp(int maxWarp) {
		this.maxWarp = maxWarp;
	}

	public int getLeft_draft_point() {
		return left_draft_point;
	}

	public void setLeft_draft_point(int leftDraftPoint) {
		left_draft_point = leftDraftPoint;
	}

	@Override
	public BaseWebTraderField packageing(List<BaseWebTraderField> packages) {
		MB13332 rt = new MB13332();
		for (BaseWebTraderField baseWebTraderField : packages) {
			MB13332 temp = (MB13332)baseWebTraderField;
			rt.tradeOrders.addAll(temp.getTradeOrders());
		}
		return rt;
	}
}
